@@ -159,9 +159,7 @@
 #include <nsIPromptService.h>
 #include <nsIWebProgressListener.h>
 #include <nsCOMPtr.h>
-#define MOZILLA_INTERNAL_API
-#include <nsIServiceManager.h>
-#undef MOZILLA_INTERNAL_API
+#include <nsServiceManagerUtils.h>
 #include <nsIInterfaceRequestorUtils.h>
 #include <nsIURI.h>
 #include <nsIURL.h>
@@ -501,11 +499,15 @@ NS_METHOD GContentHandler::SynchroniseMIMEInfo (void)
 	char * info = g_strdup_printf ("%d:%s", gtk_get_current_event_time(),
 				       id);
 
+#ifdef HAVE_NSIMIMEINFO_SETDESCRIPTION
+	rv = mimeInfo->SetDescription(GulString(info));
+#else
 	rv = mimeInfo->SetApplicationDescription
 #ifdef HAVE_NSIMIMEINFO_NSASTRING
 		(GulString(info));
 #else
 		(GulString(info).get());
+#endif
 #endif
 	g_free (info);
 
@@ -965,5 +967,15 @@ NS_IMETHODIMP MiniProgressListener::OnProgressChange64(nsIWebProgress *aWebProgr
 {
     return NS_ERROR_NOT_IMPLEMENTED;
 }
+
+#ifdef HAVE_NSIWEBPROGRESSLISTENER2_ONREFRESHATTEMPTED
+NS_IMETHODIMP MiniProgressListener::OnRefreshAttempted(nsIWebProgress *aWebProgress,
+						nsIURI *aRefreshURI, PRInt32 aMillis,
+						PRBool aSameURI, PRBool *_retval)
+{
+    return NS_ERROR_NOT_IMPLEMENTED;
+}
+#endif
+
 #endif
 
