@@ -20,9 +20,7 @@
 #endif
 
 #include <nscore.h>
-#define MOZILLA_INTERNAL_API
-#include <nsIServiceManager.h>
-#undef MOZILLA_INTERNAL_API
+#include <nsComponentManagerUtils.h>
 
 #include "EventContext.h"
 #include "GaleonWrapper.h"
