@@ -16,7 +16,6 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: FilePicker.cpp,v 1.32 2004/05/07 10:56:25 crispin Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -24,29 +23,22 @@
 #endif
 
 #include "FilePicker.h"
+#include "GaleonUtils.h"
 
-#include "nsCRT.h"
-#include "nsCOMPtr.h"
-#include "nsISupportsArray.h"
-#include "nsIServiceManager.h"
+#include <nsCOMPtr.h>
+#include <nsCOMPtr.h>
+#include <nsServiceManagerUtils.h>
+#include <nsComponentManagerUtils.h>
+#include <nsIURI.h>
+#include <nsIFileURL.h>
+#include <nsILocalFile.h>
+#include <nsIDOMWindow.h>
+#include <nsNetCID.h>
+#include <nsXPCOMCID.h>
 
-#include "nsString.h"
-#include "nsXPIDLString.h"
-#include "nsIPrefService.h"
-#include "nsIURI.h"
-#include "nsIFileURL.h"
-#include "nsIChannel.h"
-#include "nsIFileChannel.h"
-#include "nsNetCID.h"
-#include "nsILocalFile.h"
-#include "nsIPromptService.h"
-#include "nsReadableUtils.h"
-#include "nsIDOMWindow.h"
-#include "nsIDOMWindowInternal.h"
-#include "nsCOMPtr.h"
-#include "nsString.h"
-#include "nsILocalFile.h"
-#include "MozillaPrivate.h"
+#ifndef HAVE_NSIFILEPICKER_NSASTRING
+#include <nsIDOMWindowInternal.h>
+#endif
 
 #include "gul-string.h"
 #include "prefs-strings.h"
@@ -61,22 +53,14 @@
 
 NS_IMPL_ISUPPORTS1(GFilePicker, nsIFilePicker)
 
-GFilePicker::GFilePicker()
+GFilePicker::GFilePicker() : mDialog (nsnull), mMode(nsIFilePicker::modeOpen)
 {
-	LOG ("GFilePicker ctor (%p)", this)
-
-	mDialog = GUL_FILE_CHOOSER (g_object_new (GUL_TYPE_FILE_CHOOSER, NULL));
-
-	g_object_add_weak_pointer (G_OBJECT (mDialog), (gpointer *) &mDialog);
-
-	gul_file_chooser_set_persist_key (mDialog, CONF_STATE_LAST_UPLOAD_DIR);
-
-	mMode = nsIFilePicker::modeOpen;
+	LOG ("GFilePicker ctor (%p)", this);
 }
 
 GFilePicker::~GFilePicker()
 {
-	LOG ("GFilePicker dtor (%p)", this)
+	LOG ("GFilePicker dtor (%p)", this);
 
 	if (mDialog)
 	{
@@ -86,68 +70,53 @@ GFilePicker::~GFilePicker()
 }
 
 /* void init (in nsIDOMWindowInternal parent, in wstring title, in short mode); */
-#if MOZILLA_SNAPSHOT > 16
+#ifdef HAVE_NSIFILEPICKER_NSASTRING
 NS_IMETHODIMP GFilePicker::Init(nsIDOMWindow *parent, const nsAString &title, PRInt16 mode)
 #else
 NS_IMETHODIMP GFilePicker::Init(nsIDOMWindowInternal *parent, const PRUnichar *title, PRInt16 mode)
 #endif
 {
-	LOG ("GFilePicker::Init")
+	LOG ("GFilePicker::Init");
 
-	if (parent)
-	{
-		GtkWidget *pwin = MozillaFindGtkParent (parent);
-
-		gtk_window_set_transient_for (GTK_WINDOW (mDialog), GTK_WINDOW (pwin));
-	}
-
-	gtk_window_set_title (GTK_WINDOW (mDialog), NS_ConvertUCS2toUTF8 (title).get());
+	GtkWidget *gparent = GaleonUtils::FindGtkParent (parent);
 
 	mMode = mode;
 
+	GtkFileChooserAction action;
 	switch (mode)
 	{
 		case nsIFilePicker::modeGetFolder:
-			gtk_file_chooser_set_action (GTK_FILE_CHOOSER (mDialog),
-						     GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
-			
-			gtk_dialog_add_buttons (GTK_DIALOG (mDialog),
-						GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-						GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
-						NULL);
-			gtk_dialog_set_default_response (GTK_DIALOG (mDialog), GTK_RESPONSE_ACCEPT);
+			action = GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER;
 			break;
 
 		case nsIFilePicker::modeOpenMultiple:
-			gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (mDialog), TRUE);
-			/* fallthrough */			
 		case nsIFilePicker::modeOpen:
-			gtk_file_chooser_set_action (GTK_FILE_CHOOSER (mDialog),
-						     GTK_FILE_CHOOSER_ACTION_OPEN);
-
-			gtk_dialog_add_buttons (GTK_DIALOG (mDialog),
-						GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-						GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
-						NULL);
-			gtk_dialog_set_default_response (GTK_DIALOG (mDialog), GTK_RESPONSE_ACCEPT);
-
+			action = GTK_FILE_CHOOSER_ACTION_OPEN;
 			break;
 
 		case nsIFilePicker::modeSave:
-			gtk_file_chooser_set_action (GTK_FILE_CHOOSER (mDialog),
-						     GTK_FILE_CHOOSER_ACTION_SAVE);
-	
-			gtk_dialog_add_buttons (GTK_DIALOG (mDialog),
-						GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-						GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
-						NULL);
-			gtk_dialog_set_default_response (GTK_DIALOG (mDialog), GTK_RESPONSE_ACCEPT);
+			action = GTK_FILE_CHOOSER_ACTION_SAVE;
 			break;
 		default:
+			/* Avoid a warning */
+			action = GTK_FILE_CHOOSER_ACTION_SAVE;
 			g_assert_not_reached ();
 			break;
 	}
 
+
+
+	mDialog = gul_file_chooser_new (GulCString (title).get(), 
+					gparent, action,
+					CONF_STATE_LAST_UPLOAD_DIR);
+
+	g_object_add_weak_pointer (G_OBJECT (mDialog), (gpointer *) &mDialog);
+
+	if (mode == nsIFilePicker::modeOpenMultiple)
+	{
+		gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (mDialog), TRUE);
+	}
+
 	return NS_OK;
 }
 
@@ -159,65 +128,59 @@ NS_IMETHODIMP GFilePicker::AppendFilters(PRInt32 filterMask)
 
 	// FIXME: use filters with mimetypes instead of extensions
 
-	LOG ("GFilePicker::AppendFilters mask=%d", filterMask)
+        NS_ENSURE_TRUE (mDialog, NS_ERROR_FAILURE);
 
-#if MOZILLA_SNAPSHOT > 16
-#define APPEND_FILTER(name, ext) AppendFilter(name, ext)
-#else
-#define APPEND_FILTER(name, ext) AppendFilter(name.get(), ext.get())
-#endif
+	LOG ("GFilePicker::AppendFilters mask=%d", filterMask);
 
 	if (filterMask & nsIFilePicker::filterAll)
 	{
-		APPEND_FILTER (NS_ConvertUTF8toUCS2 (_("All files")),
-			      NS_LITERAL_STRING ("*"));
+		AppendFilter (_("All files"), "*");
 	}
 	if (filterMask & nsIFilePicker::filterHTML)
 	{
-		APPEND_FILTER (NS_ConvertUTF8toUCS2 (_("HTML files")),
-			      NS_LITERAL_STRING ("*.html; *.htm; *.shtml; *.xhtml"));
+		AppendFilter (_("HTML files"), "*.html; *.htm; *.shtml; *.xhtml");
 	}
 	if (filterMask & nsIFilePicker::filterText)
 	{
-		APPEND_FILTER (NS_ConvertUTF8toUCS2 (_("Text files")),
-			      NS_LITERAL_STRING ("*.txt; *.text"));
+		AppendFilter (_("Text files"), "*.txt; *.text");
 	}
 	if (filterMask & nsIFilePicker::filterImages)
 	{
-		APPEND_FILTER (NS_ConvertUTF8toUCS2 (_("Image files")),
-			      NS_LITERAL_STRING ("*.png; *.gif; *.jpeg; *.jpg"));
+		AppendFilter (_("Image files"), "*.png; *.gif; *.jpeg; *.jpg");
 	}
 	if (filterMask & nsIFilePicker::filterXML)
 	{
-		APPEND_FILTER (NS_ConvertUTF8toUCS2 (_("XML files")),
-			      NS_LITERAL_STRING ("*.xml"));
+		AppendFilter (_("XML files"), "*.xml");
 	}
 	if (filterMask & nsIFilePicker::filterXUL)
 	{
-		APPEND_FILTER (NS_ConvertUTF8toUCS2 (_("XUL files")),
-			      NS_LITERAL_STRING ("*.xul"));
+		AppendFilter (_("XUL files"), "*.xul");
 	}
-#undef APPEND_FILTER
 
 	return NS_OK;
 }
 
 /* void appendFilter (in wstring title, in wstring filter); */
-#if MOZILLA_SNAPSHOT > 16
+#ifdef HAVE_NSIFILEPICKER_NSASTRING
 NS_IMETHODIMP GFilePicker::AppendFilter(const nsAString &title, const nsAString &filter)
 #else
 NS_IMETHODIMP GFilePicker::AppendFilter(const PRUnichar *title, const PRUnichar *filter)
 #endif
 {
-	LOG ("GFilePicker::AppendFilter title '%s' for '%s'",
-	     NS_ConvertUCS2toUTF8 (title).get(),
-	     NS_ConvertUCS2toUTF8 (filter).get())
+        NS_ENSURE_TRUE (mDialog, NS_ERROR_FAILURE);
 
-	NS_ConvertUCS2toUTF8 pattern(filter);
-	pattern.StripWhitespace();
-	if (pattern.IsEmpty()) return NS_ERROR_FAILURE;
+	LOG ("GFilePicker::AppendFilter ()");
 
-	char **patterns = g_strsplit (pattern.get(), ";", -1);
+	return AppendFilter (GulCString (title).get(), GulCString (filter).get());
+}
+
+
+nsresult GFilePicker::AppendFilter (const char * title, const char * pattern)
+{
+        NS_ENSURE_TRUE (title, NS_ERROR_FAILURE);
+        NS_ENSURE_TRUE (pattern, NS_ERROR_FAILURE);
+
+	char **patterns = g_strsplit (pattern, ";", -1);
 
 	GtkFileFilter *filth = gtk_file_filter_new ();
 
@@ -226,7 +189,7 @@ NS_IMETHODIMP GFilePicker::AppendFilter(const PRUnichar *title, const PRUnichar
 		gtk_file_filter_add_pattern (filth, patterns[i]);
 	}
 
-	gtk_file_filter_set_name (filth, NS_ConvertUCS2toUTF8(title).get());
+	gtk_file_filter_set_name (filth, title);
 	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (mDialog), filth);
 
 	g_strfreev (patterns);
@@ -235,79 +198,71 @@ NS_IMETHODIMP GFilePicker::AppendFilter(const PRUnichar *title, const PRUnichar
 }
 
 /* attribute wstring defaultString; */
-#if MOZILLA_SNAPSHOT > 16
+#ifdef HAVE_NSIFILEPICKER_NSASTRING
 NS_IMETHODIMP GFilePicker::GetDefaultString(nsAString &aDefaultString)
 #else
 NS_IMETHODIMP GFilePicker::GetDefaultString(PRUnichar **aDefaultString)
 #endif
 {
-	char *filename, *converted;
+        NS_ENSURE_TRUE (mDialog, NS_ERROR_FAILURE);
 
-	LOG ("GFilePicker::GetDefaultString")
+	LOG ("GFilePicker::GetDefaultString");
 
-	filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (mDialog));
-	if (filename != NULL)
-	{
-		converted = g_filename_to_utf8(filename, -1, NULL, NULL, NULL);
-
-#if MOZILLA_SNAPSHOT > 16
-		aDefaultString.Assign(NS_ConvertUTF8toUCS2(converted));
+#ifdef HAVE_NSIFILEPICKER_NSASTRING
+	aDefaultString = mDefaultString;
 #else
-		/* FIXME: when can depend on moz >= 1.6, use CopyUTF8toUCS2 here */
-		*aDefaultString = ToNewUnicode (NS_ConvertUTF8toUCS2 (converted));
+	*aDefaultString = NS_StringCloneData (mDefaultString);
 #endif
-	
-		g_free (filename);
-		g_free (converted);
-	}
 
 	return NS_OK;
 }
 
-#if MOZILLA_SNAPSHOT > 16
+#ifdef HAVE_NSIFILEPICKER_NSASTRING
 NS_IMETHODIMP GFilePicker::SetDefaultString(const nsAString &aDefaultString)
 #else
 NS_IMETHODIMP GFilePicker::SetDefaultString(const PRUnichar *aDefaultString)
 #endif
 {
-	LOG ("GFilePicker::SetDefaultString to %s",
-	     NS_ConvertUCS2toUTF8 (aDefaultString).get())
+        NS_ENSURE_TRUE (mDialog, NS_ERROR_FAILURE);
 
-#if MOZILLA_SNAPSHOT > 16
-	if (aDefaultString.Length())
-#else
-	if (aDefaultString)
-#endif
+	LOG ("GFilePicker::SetDefaultString ()");
+
+	/* Save the default string (i.e. the filename) for the open mode
+	 * for when SetDefaultDir is called later */
+	mDefaultString.Assign (aDefaultString);
+
+	if (mMode == nsIFilePicker::modeSave)
 	{
+                if (!mDefaultString.Length()) return NS_ERROR_FAILURE;
+
 		/* set_current_name takes UTF-8, not a filename */
 		gtk_file_chooser_set_current_name
 			(GTK_FILE_CHOOSER (mDialog),
-			 NS_ConvertUCS2toUTF8 (aDefaultString).get());
+			 GulCString (mDefaultString).get());
 	}
 
 	return NS_OK;
 }
 
 /* attribute wstring defaultExtension; */
-#if MOZILLA_SNAPSHOT > 16
+#ifdef HAVE_NSIFILEPICKER_NSASTRING
 NS_IMETHODIMP GFilePicker::GetDefaultExtension(nsAString &aDefaultExtension)
 #else
 NS_IMETHODIMP GFilePicker::GetDefaultExtension(PRUnichar **aDefaultExtension)
 #endif
 {
-	LOG ("GFilePicker::GetDefaultExtension")
+	LOG ("GFilePicker::GetDefaultExtension");
 
 	return NS_ERROR_NOT_IMPLEMENTED;
 }
 
-#if MOZILLA_SNAPSHOT > 16
+#ifdef HAVE_NSIFILEPICKER_NSASTRING
 NS_IMETHODIMP GFilePicker::SetDefaultExtension(const nsAString &aDefaultExtension)
 #else
 NS_IMETHODIMP GFilePicker::SetDefaultExtension(const PRUnichar *aDefaultExtension)
 #endif
 {
-	LOG ("GFilePicker::SetDefaultExtension to %s",
-	     NS_ConvertUCS2toUTF8(aDefaultExtension).get())
+	LOG ("GFilePicker::SetDefaultExtension()");
 
 	return NS_ERROR_NOT_IMPLEMENTED;
 }
@@ -315,14 +270,14 @@ NS_IMETHODIMP GFilePicker::SetDefaultExtension(const PRUnichar *aDefaultExtensio
 /* attribute long filterIndex; */
 NS_IMETHODIMP GFilePicker::GetFilterIndex(PRInt32 *aFilterIndex)
 {
-	LOG ("GFilePicker::GetFilterIndex")
+	LOG ("GFilePicker::GetFilterIndex");
 
 	return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 NS_IMETHODIMP GFilePicker::SetFilterIndex(PRInt32 aFilterIndex)
 {
-	LOG ("GFilePicker::SetFilterIndex index=%d", aFilterIndex)
+	LOG ("GFilePicker::SetFilterIndex index=%d", aFilterIndex);
 
 	return NS_ERROR_NOT_IMPLEMENTED;
 }
@@ -330,16 +285,16 @@ NS_IMETHODIMP GFilePicker::SetFilterIndex(PRInt32 aFilterIndex)
 /* attribute nsILocalFile displayDirectory; */
 NS_IMETHODIMP GFilePicker::GetDisplayDirectory(nsILocalFile **aDisplayDirectory)
 {
-	char *dir;
+        NS_ENSURE_TRUE (mDialog, NS_ERROR_FAILURE);
 
-	LOG ("GFilePicker::GetDisplayDirectory")
+	LOG ("GFilePicker::GetDisplayDirectory");
 
-	dir = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (mDialog));
+	char *dir = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (mDialog));
 
 	if (dir != NULL)
 	{
 		nsCOMPtr<nsILocalFile> file = do_CreateInstance (NS_LOCAL_FILE_CONTRACTID);
-		file->InitWithNativePath (nsDependentCString (dir));
+		file->InitWithNativePath (GulDependentCString (dir));
 		NS_IF_ADDREF (*aDisplayDirectory = file);
 	
 		g_free (dir);
@@ -350,13 +305,30 @@ NS_IMETHODIMP GFilePicker::GetDisplayDirectory(nsILocalFile **aDisplayDirectory)
 
 NS_IMETHODIMP GFilePicker::SetDisplayDirectory(nsILocalFile *aDisplayDirectory)
 {
-	nsCAutoString dir;
+        NS_ENSURE_TRUE (mDialog, NS_ERROR_FAILURE);
+
+	GulCString dir;
 	aDisplayDirectory->GetNativePath (dir);
 
-	LOG ("GFilePicker::SetDisplayDirectory to %s", dir.get())
+	LOG ("GFilePicker::SetDisplayDirectory to %s", dir.get());
+	
+	if (mDefaultString.Length() && mMode != nsIFilePicker::modeSave)
+	{
+		nsEmbedCString defaultString;
+		/* NOTE, not UTF8, so can't use GulStrings */
+		NS_UTF16ToCString (mDefaultString, NS_CSTRING_ENCODING_NATIVE_FILESYSTEM,
+				   defaultString);
 
-	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (mDialog),
-					     dir.get());
+		char *filename = g_build_filename (dir.get(), defaultString.get(), NULL);
+		LOG ("Setting filename to %s", filename);
+                gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (mDialog), filename);
+                g_free (filename);
+	}
+	else
+	{
+		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (mDialog),
+						     dir.get());
+	}
 
 	return NS_OK;
 }
@@ -364,16 +336,16 @@ NS_IMETHODIMP GFilePicker::SetDisplayDirectory(nsILocalFile *aDisplayDirectory)
 /* readonly attribute nsILocalFile file; */
 NS_IMETHODIMP GFilePicker::GetFile(nsILocalFile **aFile)
 {
-	char *filename;
+        NS_ENSURE_TRUE (mDialog, NS_ERROR_FAILURE);
 
-	LOG ("GFilePicker::GetFile")
+	LOG ("GFilePicker::GetFile");
 
-	filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (mDialog));
+	char *filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (mDialog));
 
 	if (filename != NULL)
 	{
 		nsCOMPtr<nsILocalFile> file = do_CreateInstance (NS_LOCAL_FILE_CONTRACTID);
-		file->InitWithNativePath (nsDependentCString (filename));
+		file->InitWithNativePath (GulDependentCString (filename));
 		NS_IF_ADDREF (*aFile = file);
 	
 		g_free (filename);
@@ -385,7 +357,7 @@ NS_IMETHODIMP GFilePicker::GetFile(nsILocalFile **aFile)
 /* readonly attribute nsIFileURL fileURL; */
 NS_IMETHODIMP GFilePicker::GetFileURL(nsIFileURL **aFileURL)
 {
-	LOG ("GFilePicker::GetFileURL")
+	LOG ("GFilePicker::GetFileURL");
 
 	nsCOMPtr<nsILocalFile> file;
 	GetFile (getter_AddRefs(file));
@@ -404,7 +376,7 @@ NS_IMETHODIMP GFilePicker::GetFiles(nsISimpleEnumerator * *aFiles)
 	// Not sure if we need to implement it at all, it's used nowhere
 	// in mozilla, but I guess a javascript might call it?
 
-	LOG ("GFilePicker::GetFiles")
+	LOG ("GFilePicker::GetFiles");
 
 	return NS_ERROR_NOT_IMPLEMENTED;
 }
@@ -412,7 +384,7 @@ NS_IMETHODIMP GFilePicker::GetFiles(nsISimpleEnumerator * *aFiles)
 /* short show (); */
 NS_IMETHODIMP GFilePicker::Show(PRInt16 *_retval)
 {
-	LOG ("GFilePicker::Show")
+	LOG ("GFilePicker::Show");
 
 	gtk_window_set_modal (GTK_WINDOW (mDialog), TRUE);
 
@@ -428,7 +400,7 @@ NS_IMETHODIMP GFilePicker::Show(PRInt16 *_retval)
 		g_free (filename);
 		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (mDialog));
 
-		LOG ("GFilePicker::Show response=%d, filename=%s", response, filename)
+		LOG ("GFilePicker::Show response=%d, filename=%s", response, filename);
 	}
 	while (response == GTK_RESPONSE_ACCEPT &&
 	       mMode == nsIFilePicker::modeSave &&
