@@ -25,16 +25,15 @@
 
 #include "galeon-debug.h"
 
-#undef MOZILLA_INTERNAL_API
 #include <nsEmbedString.h>
-#define MOZILLA_INTERNAL_API 1
 
 #define MOZILLA_CLIENT
 #include <gtkmozembed.h>
 #include <gtkmozembed_internal.h>
 #undef MOZILLA_CLIENT
 #include <nsCOMPtr.h>
-#include <nsIServiceManager.h>
+#include <nsComponentManagerUtils.h>
+#include <nsServiceManagerUtils.h>
 #include <nsIInterfaceRequestorUtils.h>
 #include <nsIDOMWindow.h>
 #include <nsIWebBrowser.h>
@@ -229,10 +228,10 @@ GaleonFind::Find (const char *aSearchString,
 
   nsresult rv;
   PRUint16 found = nsITypeAheadFind::FIND_NOTFOUND;
-#ifdef HAVE_NSITYPEAHEADFIND_SETSELECTIONMODEANDREPAINT
-  rv = mFinder->Find (uSearchString, aLinksOnly, PR_FALSE, &found);
-#else
+#ifdef HAVE_NSITYPEAHEADFIND_FINDNOFOCUS
   rv = mFinder->Find (uSearchString, aLinksOnly, &found);
+#else
+  rv = mFinder->Find (uSearchString, aLinksOnly, PR_FALSE, &found);
 #endif
 
   return (GaleonEmbedFindResult) found;
@@ -250,7 +249,7 @@ GaleonFind::Find (const char *aSearchString,
 }
 
 GaleonEmbedFindResult
-GaleonFind::FindAgain (PRBool aForward)
+GaleonFind::FindAgain (PRBool aForward, PRBool aLinksOnly)
 {
   if (!mFinder) return GALEON_EMBED_FIND_NOTFOUND;
 
@@ -261,13 +260,21 @@ GaleonFind::FindAgain (PRBool aForward)
   PRUint16 found = nsITypeAheadFind::FIND_NOTFOUND;
 #ifdef HAVE_NSITYPEAHEADFIND_SETSELECTIONMODEANDREPAINT
   if (aForward) {
-#ifdef HAVE_NSITYPEAHEADFIND_FINDAGAIN
+#if defined(HAVE_NSITYPEAHEADFIND_FINDNOFOCUS)
+    rv = mFinder->FindAgain(PR_FALSE, aLinksOnly, &found);
+#elif defined(HAVE_NSITYPEAHEADFIND_FINDAGAIN_ALINKSONLY)
+    rv = mFinder->FindAgain(PR_FALSE, aLinksOnly, PR_FALSE, &found);
+#elif defined(HAVE_NSITYPEAHEADFIND_FINDAGAIN)
     rv = mFinder->FindAgain(PR_FALSE, PR_FALSE, &found);
 #else
     rv = mFinder->FindNext (PR_FALSE, &found);
 #endif
   } else {
-#ifdef HAVE_NSITYPEAHEADFIND_FINDAGAIN
+#if defined(HAVE_NSITYPEAHEADFIND_FINDNOFOCUS)
+    rv = mFinder->FindAgain(PR_TRUE, aLinksOnly, &found);
+#elif defined(HAVE_NSITYPEAHEADFIND_FINDAGAIN_ALINKSONLY)
+    rv = mFinder->FindAgain(PR_TRUE, aLinksOnly, PR_FALSE, &found);
+#elif defined(HAVE_NSITYPEAHEADFIND_FINDAGAIN)
     rv = mFinder->FindAgain(PR_TRUE, PR_FALSE, &found);
 #else
     rv = mFinder->FindPrevious (PR_FALSE, &found);
