@@ -36,15 +36,12 @@
  *
  * ***** END LICENSE BLOCK *****
  *
- * $Id: HeaderSniffer.cpp,v 1.3 2004/05/23 21:19:46 crispin Exp $
  */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#include "MozillaPrivate.h"
-
 #include "ProgressListener.h"
 #include "HeaderSniffer.h"
 #include "netCore.h"
@@ -52,29 +49,33 @@
 #include "galeon-embed-shell.h"
 #include "gul-file-chooser.h"
 #include "gul-gui.h"
+#include "gul-x11.h"
 #include "eel-gconf-extensions.h"
 #include "galeon-debug.h"
 #include "prefs-strings.h"
 #include "hig-alert.h"
 
-#include "nsReadableUtils.h"
-#include "nsIChannel.h"
-#include "nsIHttpChannel.h"
-#include "nsIEncodedChannel.h"
-#include "nsIURIChecker.h"
-#include "nsIURL.h"
-#include "nsIStringEnumerator.h"
-#include "nsIMIMEService.h"
-#include "nsIMIMEInfo.h"
-#include "nsIDOMHTMLDocument.h"
-#include "nsIDownload.h"
-#include "nsIWindowWatcher.h"
-#include "nsNetCID.h"
-#include "nsIDOMDocument.h"
-#include "nsIDOMNSDocument.h"
-#if MOZILLA_SNAPSHOT > 10
-#include  "nsIMIMEHeaderParam.h"
-#endif
+#include <nsComponentManagerUtils.h>
+#include <nsServiceManagerUtils.h>
+#include <nsIChannel.h>
+#include <nsIHttpChannel.h>
+#include <nsIEncodedChannel.h>
+#include <nsIURIChecker.h>
+#include <nsIURL.h>
+#include <nsIStringEnumerator.h>
+#include <nsIMIMEService.h>
+#include <nsIMIMEInfo.h>
+#include <nsIDOMHTMLDocument.h>
+#include <nsIWindowWatcher.h>
+#include <nsNetCID.h>
+#include <nsIDOMDocument.h>
+#include <nsIDOMNSDocument.h>
+#include <nsIMIMEHeaderParam.h>
+#include <nsIExternalHelperAppService.h>
+#include <nsCExternalHandlerService.h>
+#include <nsILocalFile.h>
+#include <nsIInputStream.h>
+#include <nsMemory.h>
 
 #include <glib/gi18n.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
@@ -128,7 +129,7 @@ HeaderSniffer::OnStopRequest (nsIRequest *aRequest,
 	{
 		GtkWidget *parent, *dialog;
 
-		galeon_embed_persist_get_fc_parent (mEmbedPersist, &parent);
+		parent = galeon_embed_persist_get_fc_parent (mEmbedPersist);
 
 		dialog = hig_alert_new (parent ? GTK_WINDOW (parent) : NULL,
 					GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -160,16 +161,16 @@ HeaderSniffer::OnStopRequest (nsIRequest *aRequest,
 	 * hint on the filename */
 	nsCOMPtr<nsIHttpChannel> httpChannel(do_QueryInterface(channel));
 
-	nsCString contentDisposition;
+	GulCString contentDisposition;
 	if (httpChannel)
 	{
-		httpChannel->GetResponseHeader(nsCAutoString("content-disposition"),
+		httpChannel->GetResponseHeader(NS_LITERAL_CSTRING("content-disposition"),
 					       contentDisposition);
 	}
 
 	/* Get the document encoding */
 	nsCOMPtr<nsIEncodedChannel> encodedChannel(do_QueryInterface(channel));
-	nsCString contentEncoding;
+	GulCString contentEncoding;
 	if (encodedChannel)
 	{
 		nsCOMPtr<nsIUTF8StringEnumerator> enumerator;
@@ -188,12 +189,12 @@ HeaderSniffer::OnStopRequest (nsIRequest *aRequest,
 	}
 
     	/* Get the Content-Type header */
-	nsCString contentType;
+	GulCString contentType;
 	channel->GetContentType(contentType);
 	
-	if (contentType == NS_LITERAL_CSTRING ("application/x-unknown-content-type"))
+	if (contentType.Equals ("application/x-unknown-content-type"))
 	{
-		contentType.SetLength( 0 );
+		contentType = "";
 	}
 
 	/* If no Content-Type, try and get it from the document */
@@ -202,10 +203,9 @@ HeaderSniffer::OnStopRequest (nsIRequest *aRequest,
 		nsCOMPtr<nsIDOMNSDocument> doc = do_QueryInterface(mDocument);
 		if (doc)
 		{
-			nsAutoString type;
+			GulString type;
 			doc->GetContentType (type);
-			contentType = NS_ConvertUCS2toUTF8 (type);
-
+			contentType = type;
 		}
 	}
 
@@ -214,15 +214,16 @@ HeaderSniffer::OnStopRequest (nsIRequest *aRequest,
 	{
 		nsCOMPtr<nsIMIMEService> mimeService (do_GetService(NS_MIMESERVICE_CONTRACTID));
 
-#if MOZILLA_SNAPSHOT < 17
-		nsXPIDLCString type;
-		rv = mimeService->GetTypeFromURI (mFinalURL, getter_Copies(type));
+#ifdef HAVE_NSIMIMEINFO_NSASTRING
+		mimeService->GetTypeFromURI (mFinalURL, contentType);
+#else
+		char *type = nsnull;
+		rv = mimeService->GetTypeFromURI (mFinalURL, &type);
 		if (NS_SUCCEEDED (rv))
 		{
 			contentType = type;
+			nsMemory::Free (type);
 		}
-#else
-		mimeService->GetTypeFromURI (mFinalURL, contentType);
 #endif
 	}
 
@@ -237,17 +238,17 @@ HeaderSniffer::OnStopRequest (nsIRequest *aRequest,
 		nsCOMPtr<nsIURL> resultURL = do_QueryInterface (mFinalURL);
 		if (resultURL)
 		{
-			nsCString extension;
+			GulCString extension;
 			resultURL->GetFileExtension (extension);
 
-#if MOZILLA_SNAPSHOT < 17
-			rv = helperService->ApplyDecodingForExtension (extension.get(),
-								       contentEncoding.get(),
-								       &mShouldDecode);
-#else
+#ifdef HAVE_NSIMIMEINFO_NSASTRING
 			rv = helperService->ApplyDecodingForExtension (extension,
 								       contentEncoding,
 								       &mShouldDecode);
+#else
+			rv = helperService->ApplyDecodingForExtension (extension.get(),
+								       contentEncoding.get(),
+								       &mShouldDecode);
 #endif
 
 			if (NS_FAILED (rv))
@@ -267,7 +268,7 @@ HeaderSniffer::OnStopRequest (nsIRequest *aRequest,
 	}
 
 
-	nsCString filename;
+	GulCString filename;
 	rv = GetFilename (contentDisposition, contentType, filename);
 	NS_ENSURE_TRUE (NS_SUCCEEDED (rv), NS_ERROR_FAILURE);
 
@@ -284,27 +285,27 @@ HeaderSniffer::OnStopRequest (nsIRequest *aRequest,
  * for a saved file.
  */
 NS_IMETHODIMP
-HeaderSniffer::GetDefaultExtension (const nsCString &aFilename,
-				    const nsCString &aContentType,
+HeaderSniffer::GetDefaultExtension (const GulCString &aFilename,
+				    const GulCString &aContentType,
 				    PRBool get_text_mime,
-				    nsCString &_retval)
+				    GulCString &_retval)
 				    
 {
 	nsresult rv;
 
-	nsCString scheme;
+	GulCString scheme;
 	mFinalURL->GetScheme (scheme);
 
 	/* Don't return default extensions for random files, or ftp */
-	if (aContentType == NS_LITERAL_CSTRING("application/octet-stream") ||
-	    scheme == NS_LITERAL_CSTRING ("ftp"))
+	if (aContentType.Equals ("application/octet-stream") ||
+	    scheme.Equals ("ftp"))
 	{
 		return NS_OK;
 	}
 
 	/* Unless asked, don't return one for text/plain, it is the
 	 * web server's default mime type */
-	if (!get_text_mime && aContentType == NS_LITERAL_CSTRING("text/plain"))
+	if (!get_text_mime && aContentType.Equals ("text/plain"))
 	{
 		return NS_OK;
 	}
@@ -313,7 +314,7 @@ HeaderSniffer::GetDefaultExtension (const nsCString &aFilename,
 	nsCOMPtr<nsIURL> url (do_CreateInstance(NS_STANDARDURL_CONTRACTID));
 	url->SetFilePath( aFilename );
 
-	nsCString extension;
+	GulCString extension;
 	url->GetFileExtension( extension );
 
 	/* See if this is an appropriate extension for this mime
@@ -322,20 +323,20 @@ HeaderSniffer::GetDefaultExtension (const nsCString &aFilename,
 	nsCOMPtr<nsIMIMEService> mimeService (do_GetService(NS_MIMESERVICE_CONTRACTID));
 
 	nsCOMPtr<nsIMIMEInfo> mimeInfo;
-#if MOZILLA_SNAPSHOT < 17
-	mimeService->GetFromTypeAndExtension (aContentType.get(), extension.get(), 
+#ifdef HAVE_NSIMIMEINFO_NSASTRING
+	mimeService->GetFromTypeAndExtension (aContentType, extension, 
 					      getter_AddRefs (mimeInfo));
 #else
-	mimeService->GetFromTypeAndExtension (aContentType, extension, 
+	mimeService->GetFromTypeAndExtension (aContentType.get(), extension.get(), 
 					      getter_AddRefs (mimeInfo));
 #endif
 	if (extension.Length() && mimeInfo)
 	{
 		PRBool exists = PR_FALSE;
-#if MOZILLA_SNAPSHOT < 17
-		mimeInfo->ExtensionExists (extension.get(), &exists);
-#else
+#ifdef HAVE_NSIMIMEINFO_NSASTRING
 		mimeInfo->ExtensionExists (extension, &exists);
+#else
+		mimeInfo->ExtensionExists (extension.get(), &exists);
 #endif
 		if (exists)
 		{
@@ -346,7 +347,7 @@ HeaderSniffer::GetDefaultExtension (const nsCString &aFilename,
 
 	/* If that didn't work, try the URI */
 	url = do_QueryInterface (mFinalURL);
-	nsCString urlExtension;
+	GulCString urlExtension;
 	if (url)
 	{
 		url->GetFileExtension (urlExtension);
@@ -355,10 +356,10 @@ HeaderSniffer::GetDefaultExtension (const nsCString &aFilename,
 	if (urlExtension.Length() && mimeInfo)
 	{
 		PRBool exists = PR_FALSE;
-#if MOZILLA_SNAPSHOT < 17
-		mimeInfo->ExtensionExists (urlExtension.get(), &exists);
-#else
+#ifdef HAVE_NSIMIMEINFO_NSASTRING
 		mimeInfo->ExtensionExists (urlExtension, &exists);
+#else
+		mimeInfo->ExtensionExists (urlExtension.get(), &exists);
 #endif
 		if (exists)
 		{
@@ -371,20 +372,21 @@ HeaderSniffer::GetDefaultExtension (const nsCString &aFilename,
 	 * primary extension if the mime type has one */
 	if( mimeInfo )
 	{
-#if MOZILLA_SNAPSHOT < 17
-		nsXPIDLCString ext;
-		rv = mimeInfo->GetPrimaryExtension (getter_Copies(ext));
-		if (NS_SUCCEEDED (rv))
-		{
-			_retval = ext;
-			return NS_OK;
-		}
-#else
+#ifdef HAVE_NSIMIMEINFO_NSASTRING
 		rv = mimeInfo->GetPrimaryExtension (_retval);
 		if (NS_SUCCEEDED (rv))
 		{
 			return NS_OK;
 		}
+#else
+		char *ext = nsnull;
+		rv = mimeInfo->GetPrimaryExtension (&ext);
+		if (NS_SUCCEEDED (rv))
+		{
+			_retval = ext;
+			nsMemory::Free (ext);
+			return NS_OK;
+		}
 #endif
 	}
 
@@ -393,30 +395,29 @@ HeaderSniffer::GetDefaultExtension (const nsCString &aFilename,
 }
 
 nsresult 
-HeaderSniffer::GetFilename (const nsCString &contentDisposition,
-			    const nsCString &contentType,
-			    nsCString &defaultFileName)
+HeaderSniffer::GetFilename (const GulCString &contentDisposition,
+			    const GulCString &contentType,
+			    GulCString &defaultFileName)
 {
-	nsresult rv;
 	PRBool fix_extension = PR_FALSE;
 	PRBool use_text_mime = PR_TRUE;
 
 	if (!contentDisposition.IsEmpty())
 	{
 		/* 1 Use the HTTP header suggestion. */
-#if MOZILLA_SNAPSHOT > 10
+		nsresult rv;
 		nsCOMPtr<nsIMIMEHeaderParam> mimehdrpar =
 			do_GetService(NS_MIMEHEADERPARAM_CONTRACTID);
 		
 		if (mimehdrpar)
 		{
-			nsCAutoString fallbackCharset;
+			GulCString fallbackCharset;
 			if (mFinalURL)
 			{
 				mFinalURL->GetOriginCharset(fallbackCharset);
 			}
 			
-			nsAutoString fileName;
+			GulString fileName;
 			
 			rv = mimehdrpar->GetParameter (contentDisposition, "filename",
 						       fallbackCharset, PR_TRUE, nsnull,
@@ -430,21 +431,9 @@ HeaderSniffer::GetFilename (const nsCString &contentDisposition,
 
 			if (NS_SUCCEEDED(rv) && !fileName.IsEmpty())
 			{
-				defaultFileName = 
-					NS_ConvertUCS2toUTF8 (fileName);
+				defaultFileName = fileName;
 			}
 		}
-#else
-		PRInt32 index = contentDisposition.Find("filename=");
-		if (index >= 0)
-		{
-			/* Take the substring following the prefix. */
-			index += strlen ("filename=");
-			nsCAutoString filename;
-			contentDisposition.Right (defaultFileName,
-						  contentDisposition.Length() - index);
-		}
-#endif
 	}
     
 	/* 2 For file URLs, use the file name. */
@@ -465,9 +454,9 @@ HeaderSniffer::GetFilename (const nsCString &contentDisposition,
 		nsCOMPtr<nsIDOMHTMLDocument> htmlDoc(do_QueryInterface(mDocument));
 		if (htmlDoc)
 		{
-			nsAutoString title;
+			GulString title;
 			htmlDoc->GetTitle (title);
-			defaultFileName = NS_ConvertUCS2toUTF8 (title);
+			defaultFileName = title;
 			fix_extension = PR_TRUE;
 		}
 	}
@@ -475,18 +464,17 @@ HeaderSniffer::GetFilename (const nsCString &contentDisposition,
 	/* 4 Use the directory */
 	if (defaultFileName.IsEmpty())
 	{
-		nsCString path;
-		mFinalURL->GetPath (path);
-		if (path.Length() && path.CharAt (path.Length()-1) == '/' )
-		{
-			path.SetLength (path.Length() - 1);
-			PRInt32 index = path.RFindChar ('/');
+		GulCString pathStr;
+		mFinalURL->GetPath (pathStr);
 
-			if (index != -1 )
+		if (pathStr.Length() && pathStr.get()[pathStr.Length()-1] == '/' )
+		{
+			pathStr.Cut (pathStr.Length()-1, 1);
+			const char *path = pathStr.get();
+			const char *sep = strrchr (path, '/');
+			if (sep)
 			{
-				index++;
-				path.Right (defaultFileName,
-					    path.Length() - index);
+				defaultFileName = sep+1;
 				fix_extension = PR_TRUE;
 			}
 		}
@@ -502,7 +490,7 @@ HeaderSniffer::GetFilename (const nsCString &contentDisposition,
 	/* 6 One last case to handle about:blank and other untitled pages. */
 	if (defaultFileName.IsEmpty())
 	{
-		defaultFileName = nsDependentCString (_("Untitled"));
+		defaultFileName = _("Untitled");
 		fix_extension = PR_TRUE;
 	}
 
@@ -513,7 +501,7 @@ HeaderSniffer::GetFilename (const nsCString &contentDisposition,
 	}
 
 	/* Get the extension */
-	nsCString extension;
+	GulCString extension;
 	GetDefaultExtension (defaultFileName, contentType,
 			     use_text_mime, extension );
 	if (extension.Length())
@@ -521,9 +509,9 @@ HeaderSniffer::GetFilename (const nsCString &contentDisposition,
 		nsCOMPtr<nsIURL> url (do_CreateInstance(NS_STANDARDURL_CONTRACTID));
 		url->SetFilePath( defaultFileName );
 
-		nsCString current;
+		GulCString current;
 		url->GetFileExtension( current );
-		if (current != extension)
+		if (!current.Equals (extension))
 		{
 			defaultFileName += ".";
 			defaultFileName += extension;
@@ -555,7 +543,7 @@ filechooser_response_cb (GulFileChooser *dialog, gint response, HeaderSniffer* s
 		nsCOMPtr<nsILocalFile> destFile = do_CreateInstance (NS_LOCAL_FILE_CONTRACTID);
 		if (destFile)
 		{
-			destFile->InitWithNativePath (nsDependentCString (filename));
+			destFile->InitWithNativePath (GulDependentCString (filename));
 
 			sniffer->InitiateDownload (destFile);
 		}
@@ -573,7 +561,7 @@ content_toggled_cb (GtkWidget *toggle, GaleonEmbedPersist *persist)
 {
 	gint flags;
 
-	galeon_embed_persist_get_flags (persist, (EmbedPersistFlags*)&flags);
+	flags = (gint)galeon_embed_persist_get_flags (persist);
 
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (toggle)))
 	{
@@ -588,12 +576,12 @@ content_toggled_cb (GtkWidget *toggle, GaleonEmbedPersist *persist)
 }
 
 nsresult
-HeaderSniffer::PerformSave (const nsCString &defaultFileName)
+HeaderSniffer::PerformSave (const GulCString &defaultFileName)
 {
 	EmbedPersistFlags flags;
 	PRBool askDownloadDest;
 
-	galeon_embed_persist_get_flags (mEmbedPersist, &flags);
+	flags = galeon_embed_persist_get_flags (mEmbedPersist);
 	askDownloadDest = (flags & EMBED_PERSIST_ASK_DESTINATION) ||
 		              eel_gconf_get_boolean (CONF_DOWNLOADING_ASK_DIR);
 		
@@ -613,7 +601,7 @@ HeaderSniffer::PerformSave (const nsCString &defaultFileName)
 	g_free (default_name);
 
 	GtkWidget *parent;
-	galeon_embed_persist_get_fc_parent (mEmbedPersist, &parent);
+	parent = galeon_embed_persist_get_fc_parent (mEmbedPersist);
 
 	if (!askDownloadDest)
 	{
@@ -628,8 +616,8 @@ HeaderSniffer::PerformSave (const nsCString &defaultFileName)
 	}
 
 	GulFileChooser *dialog;
-	const char *title = NULL;
-	galeon_embed_persist_get_fc_title (mEmbedPersist, &title);
+	const char *title;
+	title = galeon_embed_persist_get_fc_title (mEmbedPersist);
 
 	dialog = gul_file_chooser_new (title ? title: _("Save"),
 				       GTK_WIDGET (parent),
@@ -642,7 +630,7 @@ HeaderSniffer::PerformSave (const nsCString &defaultFileName)
 
 		/* Reset the flags to turn off content, the user toggles
 		 * if they want to turn it on */
-		(gint)flags &= ~EMBED_PERSIST_SAVE_CONTENT;
+		(gint&)flags &= ~EMBED_PERSIST_SAVE_CONTENT;
 		galeon_embed_persist_set_flags (mEmbedPersist, flags);
 
 		toggle = gtk_check_button_new_with_mnemonic (_("Save _with content"));
@@ -659,7 +647,16 @@ HeaderSniffer::PerformSave (const nsCString &defaultFileName)
 	NS_ADDREF (this); /* add a ref to this object, so it doesn't dissappear */
 	g_signal_connect (dialog, "response",
 			  G_CALLBACK (filechooser_response_cb), this);
-	
+
+	guint32 user_time;
+	g_object_get (mEmbedPersist, "user_time", &user_time, NULL);
+
+	if (!user_time)
+	{
+		g_warning ("No user time specified in embed persist for file save" );
+	}
+	gul_x11_window_update_user_time (GTK_WIDGET (dialog), user_time);
+
 	gtk_widget_show (GTK_WIDGET (dialog));
 	
 	g_free (filename);
