@@ -24,11 +24,16 @@
 #include "ContentHandler.h"
 #include "FilePicker.h"
 #include "GlobalHistory.h"
-#include "JSConsoleService.h"
 #include "MyportalProtocolHandler.h"
 #include "PrintingPromptService.h"
 #include "ProgressListener.h"
 #include "SideBarProxy.h"
+#include "EphyPromptService.h"
+
+#ifdef HAVE_NSIJSCONSOLESERVICE_H
+#include "JSConsoleService.h"
+#endif
+
 #ifdef HAVE_MOZILLA_PSM
 #include "GtkNSSDialogs.h"
 #include "GtkNSSKeyPairDialogs.h"
@@ -37,42 +42,47 @@
 #endif
 
 #include "ExternalProtocolService.h"
-#include "gUrlCID.h"
-#include "nsAboutRedirector.h"
-#include "nsMailtoUrl.h"
+#include "GaleonAboutModule.h"
 
+#ifdef HAVE_NSICLASSINFOIMPL_H
+#include <nsIClassInfoImpl.h>
+#endif
 #include <nsIGenericFactory.h>
 #include <nsIComponentRegistrar.h>
 #include <nsICategoryManager.h>
+#include <nsIComponentManager.h>
 #include <nsCOMPtr.h>
-#define MOZILLA_INTERNAL_API
-#include <nsIServiceManager.h>
-#undef MOZILLA_INTERNAL_API
+#include <nsXPCOM.h>
+#include <nsComponentManagerUtils.h>
+#include <nsServiceManagerUtils.h>
 #include <nsDocShellCID.h>
+#include <nsXPCOMCID.h>
 
 #include <glib.h>
 
 NS_GENERIC_FACTORY_CONSTRUCTOR(GProgressListener)
 NS_GENERIC_FACTORY_CONSTRUCTOR(GFilePicker)
-NS_GENERIC_FACTORY_CONSTRUCTOR(JSConsoleService)
 NS_GENERIC_FACTORY_CONSTRUCTOR(GContentHandler)
 NS_GENERIC_FACTORY_CONSTRUCTOR(MozGlobalHistory)
 NS_GENERIC_FACTORY_CONSTRUCTOR(GPrintingPromptService)
 NS_GENERIC_FACTORY_CONSTRUCTOR(GSidebarProxy)
 NS_DECL_CLASSINFO(GSidebarProxy)
 NS_GENERIC_FACTORY_CONSTRUCTOR(GCookiePromptService)
-NS_GENERIC_FACTORY_CONSTRUCTOR(nsAboutRedirector)
+NS_GENERIC_FACTORY_CONSTRUCTOR(EphyPromptService)
 #ifdef HAVE_MOZILLA_PSM
 NS_GENERIC_FACTORY_CONSTRUCTOR(GtkNSSDialogs)
 NS_GENERIC_FACTORY_CONSTRUCTOR(GtkNSSKeyPairDialogs)
 NS_GENERIC_FACTORY_CONSTRUCTOR(GtkNSSClientAuthDialogs)
 NS_GENERIC_FACTORY_CONSTRUCTOR(GtkNSSSecurityWarningDialogs)
 #endif
+#ifdef HAVE_NSSTRING_INTERNAL
+NS_GENERIC_FACTORY_CONSTRUCTOR(GaleonAboutModule)
 NS_GENERIC_FACTORY_CONSTRUCTOR(GMyportalProtocolHandler)
+#endif
 NS_GENERIC_FACTORY_CONSTRUCTOR(GExternalProtocolService)
-
-NS_GENERIC_FACTORY_CONSTRUCTOR(nsMailtoUrl);
-static NS_DEFINE_CID(kMailtoUrlCID, NS_MAILTOURL_CID);
+#ifdef HAVE_NSIJSCONSOLESERVICE_H
+NS_GENERIC_FACTORY_CONSTRUCTOR(JSConsoleService)
+#endif
 
 // NS_DOWNLOAD_CONTRACTID was renamed in mozilla 1.8b
 #ifndef NS_TRANSFER_CONTRACTID
@@ -106,12 +116,14 @@ static const nsModuleComponentInfo sAppComps[] = {
 		NS_TRANSFER_CONTRACTID,
 		GProgressListenerConstructor
 	},
+#ifdef HAVE_NSIJSCONSOLESERVICE_H
 	{	
 		G_JS_CONSOLESERVICE_CLASSNAME,
 		G_JSCONSOLESERVICE_CID,
 		NS_JSCONSOLESERVICE_CONTRACTID,
 		JSConsoleServiceConstructor
 	},
+#endif
 	{
 		G_FILEPICKER_CLASSNAME,
 		G_FILEPICKER_CID,
@@ -162,12 +174,14 @@ static const nsModuleComponentInfo sAppComps[] = {
 		GCookiePromptServiceConstructor
 	},
 #ifdef HAVE_MOZILLA_PSM
+#ifdef HAVE_NSIBADCERTLISTENER_H
 	{
 		GTK_NSSDIALOGS_CLASSNAME,
 		GTK_NSSDIALOGS_CID,
 		NS_BADCERTLISTENER_CONTRACTID,
 		GtkNSSDialogsConstructor
 	},
+#endif
 	{
 		GTK_NSSDIALOGS_CLASSNAME,
 		GTK_NSSDIALOGS_CID,
@@ -193,6 +207,7 @@ static const nsModuleComponentInfo sAppComps[] = {
                 GtkNSSSecurityWarningDialogsConstructor
         },
 #endif
+#ifdef HAVE_NSSTRING_INTERNAL
 	{
 		G_ABOUT_MYPORTAL_CLASSNAME,
 		G_MYPORTAL_PROTOCOL_CID,
@@ -206,17 +221,32 @@ static const nsModuleComponentInfo sAppComps[] = {
 		GMyportalProtocolHandlerConstructor
 	},
 	{
-		G_ABOUT_TOPHER_MODULE_CLASSNAME,
-		G_ABOUT_REDIRECTOR_MODULE_CID,
-		G_ABOUT_TOPHER_MODULE_CONTRACTID,
-		nsAboutRedirectorConstructor
+		GALEON_ABOUT_TOPHER_CLASSNAME,
+		GALEON_ABOUT_MODULE_CID,
+		GALEON_ABOUT_TOPHER_CONTRACTID,
+		GaleonAboutModuleConstructor
 	},
-	{       /* This MUST be last */
-		NS_MAILTOURL_CLASSNAME,
-		NS_MAILTOURL_CID,
-		NS_MAILTOURL_CONTRACTID,
-		nsMailtoUrlConstructor
-	}
+	{
+		GALEON_ABOUT_NETERROR_CLASSNAME,
+		GALEON_ABOUT_MODULE_CID,
+		GALEON_ABOUT_NETERROR_CONTRACTID,
+		GaleonAboutModuleConstructor
+	},
+#endif
+	{
+		EPHY_PROMPT_SERVICE_CLASSNAME,
+		EPHY_PROMPT_SERVICE_IID,
+		"@mozilla.org/embedcomp/prompt-service;1",
+		EphyPromptServiceConstructor
+	},
+#ifdef HAVE_NSINONBLOCKINGALERTSERVICE_H
+	{
+		EPHY_PROMPT_SERVICE_CLASSNAME,
+		EPHY_PROMPT_SERVICE_IID,
+		"@mozilla.org/embedcomp/nbalert-service;1",
+		EphyPromptServiceConstructor
+	},
+#endif /* HAVE_NSINONBLOCKINGALERTSERVICE_H */
 };
 
 #define NS_MSGCONTENTPOLICY_CONTRACTID "@mozilla.org/messenger/content-policy;1"
@@ -283,17 +313,6 @@ mozilla_register_components (void)
 	NS_GetComponentManager (getter_AddRefs (cm));
 	NS_ENSURE_TRUE (cm, FALSE);
 
-	guint numAppComps = G_N_ELEMENTS (sAppComps);
-
-	/* See if we should register the nsMailtoUrl handler */
-	PRBool isMailtoUrlReg;
-	rv = cr->IsContractIDRegistered(NS_MAILTOURL_CONTRACTID,
-					&isMailtoUrlReg);
-	if (NS_SUCCEEDED(rv) && isMailtoUrlReg)
-	{
-		numAppComps--; /* No */
-	}
-
 	for (guint i = 0; i < G_N_ELEMENTS (sAppComps); i++)
 	{
 		nsCOMPtr<nsIGenericFactory> componentFactory;
