@@ -22,11 +22,10 @@
 #include <config.h>
 #endif
 
+#include <nscore.h>
+#include <nsComponentManagerUtils.h>
 #include <nsCOMPtr.h>
 #include <nsIIOService.h>
-#define MOZILLA_INTERNAL_API
-#include <nsIServiceManager.h>
-#undef MOZILLA_INTERNAL_API
 #include <nsIURI.h>
 #include <nsIChannel.h>
 #include <nsIOutputStream.h>
@@ -171,6 +170,16 @@ GMyportalProtocolHandler::AllowPort(PRInt32 port,
 	return NS_OK;
 }
 
+#ifdef HAVE_NSIABOUTMODULE_GETURIFLAGS
+/* unsigned long getURIFlags (in nsIURI aURI); */
+NS_IMETHODIMP
+GMyportalProtocolHandler::GetURIFlags(nsIURI *aURI,
+                                      PRUint32 *_retval)
+{
+	*_retval = 0;
+	return NS_OK;
+}
+#endif
 
 // ------------------------------------------------------------
 /**
@@ -467,12 +476,16 @@ GMyportalProtocolHandler::CreateMyportalPage (const char *path,
 	rv = sStream->NewInputStream(0, getter_AddRefs(iStream));
 	NS_ENSURE_SUCCESS (rv, rv);
 
-	nsCOMPtr<nsIInputStreamChannel> channel =
+	nsCOMPtr<nsIInputStreamChannel> isc =
 		do_CreateInstance(kInputStreamChannelCID, &rv);
 	NS_ENSURE_SUCCESS (rv, rv);
 
-        rv |= channel->SetURI (aURI);
-	rv |= channel->SetContentStream (iStream);
+        rv |= isc->SetURI (aURI);
+	rv |= isc->SetContentStream (iStream);
+
+	nsCOMPtr<nsIChannel> channel = do_QueryInterface (isc, &rv);
+	NS_ENSURE_SUCCESS (rv, rv);
+
         rv |= channel->SetContentType (NS_LITERAL_CSTRING ("application/xhtml+xml"));
         rv |= channel->SetContentCharset (NS_LITERAL_CSTRING ("utf-8"));
 	NS_ENSURE_SUCCESS (rv, rv);
