@@ -21,9 +21,8 @@
 #endif
 
 #include <nscore.h>
-#define MOZILLA_INTERNAL_API
-#include <nsIServiceManager.h>
-#undef MOZILLA_INTERNAL_API
+#include <nsComponentManagerUtils.h>
+#include <nsServiceManagerUtils.h>
 
 #define MOZILLA_CLIENT
 #include "gtkmozembed.h"
@@ -54,7 +53,6 @@
 #include <string.h>
 
 #include <nsIConsoleService.h>
-#include <nsIJSConsoleService.h>
 #include <nsICacheService.h>
 #include <nsCOMPtr.h>
 #include <nsNetCID.h>
@@ -65,8 +63,10 @@
 #include <nsIPermissionManager.h>
 #include <nsICookieManager.h>
 #include <nsIPermission.h>
+#ifdef HAVE_NSIPASSWORD_H
 #include <nsIPasswordManager.h>
 #include <nsIPassword.h>
+#endif
 #include <nsICookie.h>
 #include <nsCPasswordManager.h>
 #include <nsIObserverService.h>
@@ -757,6 +757,7 @@ static GList *
 impl_list_passwords (GaleonEmbedShell *shell,
 		     PasswordType type)
 {
+#ifdef HAVE_NSIPASSWORD_H
         nsresult result = NS_ERROR_FAILURE;
 
         nsCOMPtr<nsIPasswordManager> passwordManager =
@@ -797,8 +798,10 @@ impl_list_passwords (GaleonEmbedShell *shell,
 
 		passwords = g_list_prepend (passwords, p);
         }       
-
 	return g_list_reverse (passwords);
+#else
+	return NULL;
+#endif
 }
 
 static void
