@@ -1,27 +1,20 @@
 /* General */
 #define CONF_GENERAL_HOMEPAGE "/apps/galeon/Browsing/General/startpage"
-#define CONF_GENERAL_STARTPAGE_TYPE "/apps/galeon/Browsing/General/startpage_type"
 #define CONF_GENERAL_ALWAYS_SAVE_SESSION "/apps/galeon/Browsing/General/always_save_session"
 #define CONF_GENERAL_NEWPAGE_TYPE "/apps/galeon/Browsing/General/newpage_type"
+#define CONF_GENERAL_OFFLINE "/apps/galeon/Browsing/General/offline"
 
 /* Bookmarks */
-#define CONF_BOOKMARKS_TITLE "/apps/galeon/Browsing/Bookmarks/bookmark_title"
-#define CONF_BOOKMARKS_XITEMS "/apps/galeon/Browsing/Bookmarks/bm_extra_items_location"
 #define CONF_BOOKMARKS_HISTORY "/apps/galeon/Browsing/Bookmarks/smart_bm_history"
-#define CONF_BOOKMARKS_TOOLTIPS "/apps/galeon/Browsing/Bookmarks/bm_tooltips"
 #define CONF_BOOKMARKS_SMARTBOOKMARK_HIDE_ARROWS "/apps/galeon/Browsing/Bookmarks/smartbookmarks_hide_arrows"
-#define CONF_BOOKMARKS_REMOVE_CONFIRMATION_MIN "/apps/galeon/Browsing/Bookmarks/remove_confirmation_min"
-#define CONF_BOOKMARKS_AB_ENABLED "/apps/galeon/Browsing/Bookmarks/autobookmarks_enabled"
-#define CONF_BOOKMARKS_AB_COUNT "/apps/galeon/Browsing/Bookmarks/autobookmarks_count"
-#define CONF_BOOKMARKS_AB_SHORTEN "/apps/galeon/Browsing/Bookmarks/autobookmarks_shorten"
-#define CONF_BOOKMARKS_SHORTTEXT "/apps/galeon/Browsing/Bookmarks/shorttext"
+#define CONF_BOOKMARKS_EDITOR_SPLIT_VIEW "/apps/galeon/Browsing/Bookmarks/Editor/split_view"
+#define CONF_BOOKMARKS_EDITOR_EDIT_PANE "/apps/galeon/Browsing/Bookmarks/Editor/edit_pane"
+#define CONF_BOOKMARKS_MANY_BOOKMARKS_WARNING_THRESHOLD "/apps/galeon/Browsing/Bookmarks/many_bookmarks_warning_threshold"
 
 /* Autocompletion */
 #define CONF_HISTORY_AUTOCOMP_ENABLE "/apps/galeon/Browsing/History/completion_enabled"
 #define CONF_COMPLETION_SHOW_LIST_AUTO "/apps/galeon/Browsing/History/completion_show_list_auto"
-#define CONF_COMPLETION_MAX_SHORT "/apps/galeon/Browsing/History/completion_max_short"
 #define CONF_COMPLETION_SHOW_TITLES "/apps/galeon/Browsing/History/completion_show_titles"
-#define CONF_COMPLETION_HUGE_WINDOW "/apps/galeon/Browsing/History/completion_huge"
 
 /* Tabs */
 #define CONF_TABS_TABBED "/apps/galeon/UI/Tabs/tabbed"
@@ -44,25 +37,19 @@
 #define CONF_WINDOWS_SHOW_TOOLBARS "/apps/galeon/UI/Windows/show_toolbars"
 #define CONF_WINDOWS_SHOW_BOOKMARKS "/apps/galeon/UI/Windows/show_bookmarks"
 #define CONF_WINDOWS_SHOW_STATUSBAR "/apps/galeon/UI/Windows/show_statusbar"
-#define CONF_WINDOWS_SHOW_DOCK "/apps/galeon/UI/Windows/show_dock"
-#define CONF_TOOLBAR_SPINNER_THEME "/apps/galeon/UI/Toolbar/spinner_theme"
-#define CONF_TOOLBAR_SETUP "/apps/galeon/UI/Toolbar/setup"
-#define CONF_TOOLBAR_STYLE "/apps/galeon/UI/Toolbar/toolbar_style"
+#define CONF_TOOLBAR_STYLE "/apps/galeon/UI/Toolbar/toolbar_override_style"
 
 /* External apps */
-#define CONF_PROGRAMS_GNOME_MAILER "/apps/galeon/Handlers/Programs/gnome_mailer"
-#define CONF_PROGRAMS_MAILER "/apps/galeon/Handlers/Programs/mailer"
 #define CONF_DOWNLOADING_ASK_DIR "/apps/galeon/Handlers/Downloading/ask_for_download_dir"
 #define CONF_DOWNLOADING_SHOW_DETAILS "/apps/galeon/Handlers/Downloading/show_details"
 #define CONF_DOWNLOADING_EXTERNAL_DOWNLOADER "/apps/galeon/Handlers/Downloading/external_downloader"
 #define CONF_DOWNLOADING_EXTERNAL_COMMAND "/apps/galeon/Handlers/Downloading/external_download_command"
 #define CONF_DOWNLOADING_EXTERNAL_TERMINAL "/apps/galeon/Handlers/Downloading/external_download_terminal"
-#define CONF_STATE_LAST_DOWNLOAD_DIR "/apps/galeon/State/last_download_dir"
 #define CONF_DOWNLOADING_DIR "/apps/galeon/Handlers/Downloading/download_dir"
 
 /* State */ 
-#define CONF_STATE_SAVE_DIR           "/apps/galeon/State/savedir"
-#define CONF_STATE_SAVE_IMAGE_DIR     "/apps/galeon/State/saveimagedir"
+#define CONF_STATE_LAST_DOWNLOAD_DIR "/apps/galeon/State/last_download_dir"
+#define CONF_STATE_LAST_UPLOAD_DIR "/apps/galeon/State/last_upload_dir"
 #define CONF_STATE_OPEN_DIR           "/apps/galeon/State/opendir"
 
 /* mouse */
